@@ -36,7 +36,8 @@ class NVMeConnectorTestCase(test_connector.ConnectorTestCase):
     def setUp(self):
         super(NVMeConnectorTestCase, self).setUp()
         self.connector = nvme.NVMeConnector(None,
-                                            execute=self.fake_execute)
+                                            execute=self.fake_execute,
+                                            use_multipath=False)
 
     def _nvme_list_cmd(self, *args, **kwargs):
         return FAKE_NVME_LIST_OUTPUT, None
@@ -48,9 +49,10 @@ class NVMeConnectorTestCase(test_connector.ConnectorTestCase):
         self.assertEqual(expected, actual)
 
     @mock.patch.object(nvme.NVMeConnector, '_execute')
-    def test_get_nvme_devices_raise(self, mock_execute):
+    @mock.patch('time.sleep')
+    def test_get_nvme_devices_raise(self, mock_sleep, mock_execute):
         mock_execute.side_effect = putils.ProcessExecutionError
-        self.assertRaises(putils.ProcessExecutionError,
+        self.assertRaises(exception.CommandExecutionFailed,
                           self.connector._get_nvme_devices)
 
     @mock.patch.object(nvme.NVMeConnector, '_get_nvme_devices')
@@ -83,14 +85,15 @@ class NVMeConnectorTestCase(test_connector.ConnectorTestCase):
             run_as_root=True)
 
     @mock.patch.object(nvme.NVMeConnector, '_execute')
-    def test_connect_volume_raise(self, mock_execute):
+    @mock.patch('time.sleep')
+    def test_connect_volume_raise(self, mock_sleep, mock_execute):
         connection_properties = {'target_portal': 'portal',
                                  'target_port': 1,
                                  'nqn': 'nqn.volume_123',
                                  'device_path': '',
                                  'transport_type': 'rdma'}
         mock_execute.side_effect = putils.ProcessExecutionError
-        self.assertRaises(putils.ProcessExecutionError,
+        self.assertRaises(exception.CommandExecutionFailed,
                           self.connector.connect_volume,
                           connection_properties)
 
@@ -111,13 +114,16 @@ class NVMeConnectorTestCase(test_connector.ConnectorTestCase):
                           self.connector.connect_volume,
                           connection_properties)
 
+    @mock.patch.object(nvme.NVMeConnector, '_get_nvme_devices')
     @mock.patch.object(nvme.NVMeConnector, '_execute')
-    def test_disconnect_volume(self, mock_execute):
+    @mock.patch('time.sleep')
+    def test_disconnect_volume(self, mock_sleep, mock_execute, mock_devices):
         connection_properties = {'target_portal': 'portal',
                                  'target_port': 1,
                                  'nqn': 'nqn.volume_123',
-                                 'device_path': '',
+                                 'device_path': '/dev/nvme0n1',
                                  'transport_type': 'rdma'}
+        mock_devices.return_value = '/dev/nvme0n1'
         self.connector.disconnect_volume(connection_properties, None)
 
         mock_execute.asert_called_once_with(
@@ -126,13 +132,17 @@ class NVMeConnectorTestCase(test_connector.ConnectorTestCase):
             root_helper=None,
             run_as_root=True)
 
+    @mock.patch.object(nvme.NVMeConnector, '_get_nvme_devices')
     @mock.patch.object(nvme.NVMeConnector, '_execute')
-    def test_disconnect_volume_raise(self, mock_execute):
+    @mock.patch('time.sleep')
+    def test_disconnect_volume_raise(
+            self, mock_sleep, mock_execute, mock_devices):
         mock_execute.side_effect = putils.ProcessExecutionError
+        mock_devices.return_value = '/dev/nvme0n1'
         connection_properties = {'target_portal': 'portal',
                                  'target_port': 1,
                                  'nqn': 'nqn.volume_123',
-                                 'device_path': '',
+                                 'device_path': '/dev/nvme0n1',
                                  'transport_type': 'rdma'}
 
         self.assertRaises(putils.ProcessExecutionError,
@@ -153,9 +163,11 @@ class NVMeConnectorTestCase(test_connector.ConnectorTestCase):
                           self.connector.extend_volume,
                           connection_properties)
 
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'find_multipath_device_path')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'extend_volume')
     @mock.patch.object(nvme.NVMeConnector, 'get_volume_paths')
-    def test_extend_volume(self, mock_volume_paths, mock_scsi_extend):
+    def test_extend_volume(self, mock_volume_paths, mock_scsi_extend,
+                           mock_scsi_find_mpath):
         fake_new_size = 1024
         mock_volume_paths.return_value = ['/dev/vdx']
         mock_scsi_extend.return_value = fake_new_size
@@ -166,3 +178,4 @@ class NVMeConnectorTestCase(test_connector.ConnectorTestCase):
                                  'transport_type': 'rdma'}
         new_size = self.connector.extend_volume(connection_properties)
         self.assertEqual(fake_new_size, new_size)
+        mock_scsi_find_mpath.assert_not_called()
